/* This test driver program reads an encoded record from a file         */
/* and decodes it and displays the results.  It then reencodes the      */
/* record and writes it out to a different file (message2.xml) for      */
/* comparison with the original file.                                   */

#include <stdio.h>
#include <stdlib.h>
#include "rtxsrc/OSRTFileInputStream.h"
#include "rtxmlsrc/rtXmlCppMsgBuf.h"
#include "csta.h"

static int encode (DeflectCall_ELEM* pDeflectCall, OSBOOL trace, OSBOOL verbose);

void* pAppInfo = 0;

int main (int argc, char** argv)
{
   int		i, stat;
   const char*  filename = "message.xml";
   OSBOOL       trace = TRUE, verbose = FALSE;

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-i")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer [ -v ] [ -i <filename>\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -i <filename>  read encoded msg from <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   OSRTContext* pctxt = new OSRTContext();
   stat = pctxt->setRunTimeKey(_rtkey, sizeof(_rtkey));
   if (0 != stat) {
      pctxt->printErrorInfo();
      return stat;
   }
   DeflectCall_ELEM value;
   OSRTFileInputStream in (pctxt, filename);
   if (pctxt->getPtr()->keylen == 0) return -1;
   OSXMLDecodeBuffer decodeBuffer (in);
   DeflectCall_CC pdu (decodeBuffer, value);

   decodeBuffer.setDiag (verbose);

   // Decode

   stat = pdu.decode();

   pAppInfo = decodeBuffer.getAppInfo ();

   if (0 == stat) {
      if (trace) {
         printf ("decoded XML message:\n");
         pdu.print ("deflect-call");
         printf ("\n");
      }
   }
   else {
      printf ("Decoding failed\n");
      decodeBuffer.printErrorInfo();
      return stat;
   }

   // Reencode and write result to message2.xml

   if ((stat = encode (&value, trace, verbose)) != 0) {
      printf ("Reencode PDU failed; stat = %d\n", stat);
   }

   return stat;
}

/* This function encodes the given record and writes the result to      */
/* message2.xml                                                         */

static int encode (DeflectCall_ELEM* pdata, OSBOOL trace, OSBOOL verbose)
{
   int stat = 0;
   const char* filename = "message2.xml";
   const OSOCTET* msgptr;

   if (trace) {
      printf ("\n\n");
      printf ("Reencoding message..\n");
   }

   OSXMLEncodeBuffer buffer;
   DeflectCall_CC pdu (buffer, *pdata);

   buffer.setDiag (verbose);

   buffer.setAppInfo (pAppInfo);

   // Encode

   stat = pdu.encode();

   if (0 == stat) {
      if (trace) {
         msgptr = buffer.getMsgPtr();
         printf ("encoded XML message:\n");
         printf ((const char*)msgptr);
         printf ("\n");
      }
   }
   else {
      printf ("Encoding failed\n");
      buffer.printErrorInfo();
      return stat;
   }
  
   // Write the encoded message out to the output file

   buffer.write (filename);

   return stat;
}
